# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Deduplicate any number of csv file with optional column group indexing.
"""

import logging
import pandas
import sys

from csvpandas import utils

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        'csv',
        nargs='+',
        help='CSV tabular blast file of query and subject hits.')

    # common outputs
    parser.add_argument(
        '-o', '--out', metavar='FILE',
        default=sys.stdout, type=utils.opener('w'),
        help="Classification results.")

    parser.add_argument(
        '--limit', type=int, help='Limit number of rows read from each csv.')
    parser.add_argument(
        '--on',
        metavar='COLS',
        help=('Comma delimited list of column '
              'names or indices if --no-header'))
    parser.add_argument(
        '--no-header',
        action='store_true',
        help='If no header available.')
    parser.add_argument(
        '--take-last',
        action='store_true',
        help='Take the last duplicate value. Default is first.')
    parser.add_argument(
        '--stack',
        action='store_true',
        help=('keep all lines, do not deduplicate'))


def action(args):
    # for debugging:
    # pandas.set_option('display.max_columns', None)
    # pd.set_option('display.max_rows', None)

    df = []
    for csv in args.csv:
        df.append(utils.read_csv(
            csv,
            dtype=str,
            nrows=args.limit,
            comment='#',
            na_filter=False,
            header=None if args.no_header else 0))

    df = pandas.concat(df, ignore_index=True)

    if not args.stack:
        if args.on:
            on = args.on.split(',')

            if args.no_header:
                on = map(int, on)

            df = df.groupby(by=on, sort=False)
            df = df.tail(1) if args.take_last else df.head(1)
        else:
            df = df.drop_duplicates(take_last=args.take_last)

    df.to_csv(args.out, index=False)



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Convert a csv file in various ways
"""

import logging
import pandas
import sys

from csvpandas import utils

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        'csv',
        metavar='CSV',
        nargs='+',
        help='tabulated input file')

    # common outputs
    parser.add_argument(
        '-o', '--out', metavar='FILE',
        default=sys.stdout, type=utils.opener('w'),
        help="out delimited file")

    parser.add_argument(
        '--limit', type=int, help='Limit number of rows read from each csv')
    parser.add_argument(
        '--comment', metavar='CHAR',
        help=('comment character'))

    parser.add_argument(
        '-f', '--from-delimiter',
        metavar='DELIM',
        help=('Delimiter value coming in'))
    parser.add_argument(
        '-t', '--to-delimiter',
        metavar='DELIM',
        help='delimeter value coming out')

    parser.add_argument(
        '--header',
        help=('comma delimted list of header names. '
              'If used in combination with --no-header '
              'then a header wil be added.  Else '
              'this will replace the header line.'))
    parser.add_argument(
        '--no-header',
        action='store_true',
        help='if first line is data')


def action(args):
    # for debugging:
    # pandas.set_option('display.max_columns', None)
    # pd.set_option('display.max_rows', None)

    dfs = []
    for csv in args.csv:
        df = pandas.read_csv(
            csv,
            dtype=str,
            nrows=args.limit,
            sep=args.from_delimiter,
            comment=args.comment,
            na_filter=False,
            header=None if args.no_header else 0)
        dfs.append(df)

    df = pandas.concat(dfs, ignore_index=True)

    if args.header:
        df.columns = args.header.split(',')

    df.to_csv(
        args.out,
        header=args.header or not args.no_header,
        sep=args.to_delimiter,
        index=False)



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Deduplicate any number of csv file with optional column group indexing.

FIXME: This might be better off as an extension of
pandas.core.format.TableFormatter.  Going from a DataFrame to string using
apply is a bit slow.
"""

import logging
import pandas
import sys

from csvpandas import utils

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        'csv',
        nargs='*',
        default=[sys.stdin],
        help='CSV tabular blast file of query and subject hits.')

    # common outputs
    parser.add_argument(
        '-o', '--out', metavar='FILE',
        default=sys.stdout, type=utils.opener('w'),
        help="Classification results.")

    parser.add_argument(
        '--limit', type=int, help='Limit number of rows read from each csv.')
    parser.add_argument(
        '--no-header',
        action='store_true',
        help='If no header available.')


def action(args):
    # for debugging:
    # pandas.set_option('display.max_columns', None)
    # pd.set_option('display.max_rows', None)

    df = []
    for csv in args.csv:
        df.append(pandas.read_csv(
            csv,
            dtype=str,
            nrows=args.limit,
            comment='#',
            na_filter=False,
            header=None if args.no_header else 0))

    df = pandas.concat(df, ignore_index=True)

    header = df.columns
    df.columns = ['column{}'.format(col) for col in xrange(len(df.columns))]
    header_dict = {col: header[i] for i, col in enumerate(df.columns)}

    # calculate column widths
    widths = {col: df[col].map(len).max() for col in df.columns}
    # check if header is longer
    for col, width in widths.items():
        widths[col] = max(len(str(header_dict[col])), width)

    # divider - add two spaces
    divider = '+'.join('-' * (widths[col] + 2) for col in df.columns)
    divider = '|' + divider + '|\n'

    # create column formats
    cols = ['{' + col + ':<' + str(widths[col]) + '}' for col in df.columns]
    # join columns into a row
    row = '| ' + ' | '.join(cols) + ' |\n'

    # write
    args.out.write(divider)

    if not args.no_header:
        args.out.write(row.format(**header_dict))
        args.out.write(divider)

    df.apply(lambda x: args.out.write(row.format(**x)), axis=1)

    args.out.write(divider)



import version

from setuptools import setup, find_packages

setup(author='Chris Rosenthal',
      author_email='crosenth@gmail.com',
      description='Interface to manipulate csv files as Pandas DataFrames.',
      name='csvpandas',
      packages=find_packages(exclude=['tests']),
      entry_points={'console_scripts': {'csvpandas = csvpandas:main'}},
      version=version.version(),
      url='https://github.com/crosenth/csvpandas',
      install_requires=['pandas>=0.17'],
      license='GPLv3',
      classifiers=[
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Development Status :: 1 - Planning'
          ]
      )
